@@ -1,4 +1,4 @@
-//  $Id: f_cisco.cc 1.10.1.5.1.50 Thu, 20 Apr 2000 20:44:19 -0700 cengiz $
+//  $Id: f_cisco.cc 1.10.1.5.1.50.1.4 Mon, 09 Apr 2001 10:38:04 -0700 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -55,6 +55,8 @@ bool CiscoConfig::forcedInboundMatchIP = true;
 bool CiscoConfig::useAclCaches = true;
 bool CiscoConfig::compressAcls = true;
 bool CiscoConfig::usePrefixLists = false;
+bool CiscoConfig::eliminateDupMapParts = false;
+bool CiscoConfig::forceTilda = false;
 int  CiscoConfig::mapIncrements = 1;
 int  CiscoConfig::mapCount = 1;
 int  CiscoConfig::mapNumbersStartAt = 1;
@@ -391,7 +393,10 @@ void CiscoConfig::printREASno(ostream& out, const RangeList &no) {
       put_par = ! no.ranges.isSingleton() || pi->high != pi->low;
       if (put_par)
 	 if (reSplittable)
-	    out << "@";
+	    if (inTilda)
+	       out << "&";
+	    else
+	       out << "@";
 	 else
 	    out << "("; 
 
@@ -411,8 +416,29 @@ void CiscoConfig::printREASno(ostream& out, const RangeList &no) {
 }
 
 int CiscoConfig::printRE_(ostream& os, const regexp& r) {
+   static bool inAlternate = false;
    int flag = 0;
 
+   if (typeid(r) == typeid(regexp_or)) {
+      bool save = inAlternate;
+
+      if (!inAlternate) {
+	 inAlternate = true;
+	 os << "(";
+      }
+
+      flag = CiscoConfig::printRE_(os, *((regexp_or &) r).left);
+      os << "|";
+      flag &= CiscoConfig::printRE_(os, *((regexp_or &) r).right);
+
+      inAlternate = save;
+      if (!inAlternate)
+	 os  << ")";
+      return flag;
+   }
+
+   inAlternate = false;
+
    if (typeid(r) == typeid(regexp_bol)) {
       os << "^";
       return flag;
@@ -430,7 +456,9 @@ int CiscoConfig::printRE_(ostream& os, const regexp& r) {
 
    if (typeid(r) == typeid(regexp_star)) {
       bool save = reSplittable;
-      reSplittable = false;
+      reSplittable = forceTilda;
+      hasTilda = forceTilda;
+      inTilda = forceTilda;
       os << "(";
       flag = CiscoConfig::printRE_(os, *((regexp_star &) r).left);
       os << ")*";
@@ -440,7 +468,9 @@ int CiscoConfig::printRE_(ostream& os, const regexp& r) {
 
    if (typeid(r) == typeid(regexp_plus)) {
       bool save = reSplittable;
-      reSplittable = false;
+      reSplittable = forceTilda;
+      hasTilda = forceTilda;
+      inTilda = forceTilda;
       os << "(";
       flag = CiscoConfig::printRE_(os, *((regexp_plus &) r).left);
       os << ")+";
@@ -448,6 +478,26 @@ int CiscoConfig::printRE_(ostream& os, const regexp& r) {
       return flag;
    }
 
+   if (typeid(r) == typeid(regexp_tildastar)) {
+      hasTilda = true;
+      inTilda = true;
+      os << "(";
+      flag = CiscoConfig::printRE_(os, *((regexp_star &) r).left);
+      os << ")*";
+      inTilda = false;
+      return flag;
+   }
+
+   if (typeid(r) == typeid(regexp_tildaplus)) {
+      hasTilda = true;
+      inTilda = true;
+      os << "(";
+      flag = CiscoConfig::printRE_(os, *((regexp_plus &) r).left);
+      os << ")+";
+      inTilda = false;
+      return flag;
+   }
+
    if (typeid(r) == typeid(regexp_question)) {
       os << "(";
       flag = CiscoConfig::printRE_(os, *((regexp_question &) r).left);
@@ -461,15 +511,6 @@ int CiscoConfig::printRE_(ostream& os, const regexp& r) {
       return flag;
    }
 
-   if (typeid(r) == typeid(regexp_or)) {
-      os << "(";
-      flag = CiscoConfig::printRE_(os, *((regexp_or &) r).left);
-      os << "|";
-      flag &= CiscoConfig::printRE_(os, *((regexp_or &) r).right);
-      os  << ")";
-      return flag;
-   }
-
    assert(1);
    os << "REGEXP_UNKNOWN";
    return flag;
@@ -481,18 +522,28 @@ void CiscoConfig::printRE(ostream &s, const regexp &r, int aclID, bool permit){
        << aclID 
        << (permit ? " permit " : " deny ");
    reSplittable = true;
+   hasTilda = false;
+   inTilda = false;
    if (!CiscoConfig::printRE_(out, r)) // true if the expression was missing $
       out << "_";
    out << "\n";
    out << ends;
 
    int lineLen = strlen(out.str());
-   if (lineLen < 250) {
+   if (lineLen < 250 & ! hasTilda) {
       char *p = out.str();
       for (char *q = strchr(p, '@'); q; q = strchr(q, '@'))
 	 *q = '(';
       s << out.str();
    } else { // need to split into multiple lines
+      if (hasTilda) {
+	 char *p = out.str();
+	 for (char *q = strchr(p, '@'); q; q = strchr(q, '@'))
+	    *q = '(';
+	 for (char *q = strchr(p, '&'); q; q = strchr(q, '&'))
+	    *q = '@';
+      }
+
       char *p = out.str();
       char *q, *r2;
       char *r = NULL;
@@ -510,7 +561,7 @@ void CiscoConfig::printRE(ostream &s, const regexp &r, int aclID, bool permit){
 	 s << out.str();
 	 cerr << "Warning: ip as-path access-list is too long for cisco to handle" << endl;
       } else {
-	 int inc = (250 - lineLen + size) >? 5;
+	 int inc = hasTilda ? 1 : (250 - lineLen + size) >? 5;
 	 p = out.str();
 	 q = strchr(r, ')') + 1;
 	 *r = 0;
@@ -520,8 +571,8 @@ void CiscoConfig::printRE(ostream &s, const regexp &r, int aclID, bool permit){
 	    if (r2 > q - 2)
 	       r2 = q - 1;
 	    else
-	       while (*r2 != '|' && r2 > r)
-		  r2--;
+	       while (*r2 != '|' && r2 < q - 1)
+		  r2++;
 	    *r2 = 0;
 	    s << p << "(" << r << ")" << q;	
 	    r = r2 + 1;
@@ -656,20 +707,15 @@ ListOf2Ints* CiscoConfig::printASPaths(regexp_nf& path) {
 }
    
 inline void CiscoConfig::printCommunity(ostream &os, unsigned int i) {
-   switch (i) {
-   case COMMUNITY_INTERNET:
+   if (i == COMMUNITY_INTERNET)
       os << "internet";
-      break;
-   case COMMUNITY_NO_EXPORT:
+   else if (i == COMMUNITY_NO_EXPORT)
       os << "no-export";
-      break;
-   case COMMUNITY_NO_EXPORT_SUBCONFED:
+   else if (i == COMMUNITY_NO_EXPORT_SUBCONFED)
       os << "no-export-subconfed";
-      break;
-   case COMMUNITY_NO_ADVERTISE:
+   else if (i == COMMUNITY_NO_ADVERTISE)
       os << "no-advertise";
-      break;
-   default :
+   else {
       int high = i >> 16;
       int low  = i & 0xFFFF;
       if (high == 0 || high == 0xFFFF)
@@ -903,7 +949,7 @@ void CiscoConfig::exportP(ASt asno, IPAddr *addr,
 			 ASt peerAS, IPAddr *peer_addr) {
 
    // Made asno part of the map name if it's not changed by users
-   sprintf(mapName, mapNameFormat, asno, mapCount++);
+   sprintf(mapName, mapNameFormat, peerAS, mapCount++);
 
    // get the aut-num object
    const AutNum *autnum = irr->getAutNum(asno);
@@ -922,10 +968,24 @@ void CiscoConfig::exportP(ASt asno, IPAddr *addr,
 	   << " has no export policy for AS" << peerAS << endl;
 
    NormalExpression *ne;
+   NormalExpression done;
    int last = 0;
    for (; fa && !last; fa = itr.next()) {
       ne = NormalExpression::evaluate(fa->filter, peerAS);
-      last = print(ne, fa->action, EXPORT);
+
+      if (eliminateDupMapParts) {
+	 NormalExpression *ne2;
+	 ne2 = new NormalExpression(*ne);
+	 ne2->do_not();
+	 done.do_and(*ne2);
+
+	 if (! done.isEmpty())
+	    last = print(ne, fa->action, EXPORT);
+
+	 done.do_or(*ne);
+      } else
+	 last = print(ne, fa->action, EXPORT);
+
       delete ne;
    }
 
@@ -939,7 +999,7 @@ void CiscoConfig::importP(ASt asno, IPAddr *addr,
 			 ASt peerAS, IPAddr *peer_addr) {
 
    // Made asno part of the map name if it's not changed by users
-   sprintf(mapName, mapNameFormat, asno, mapCount++);
+   sprintf(mapName, mapNameFormat, peerAS, mapCount++);
 
    // get the aut-num object
    const AutNum *autnum = irr->getAutNum(asno);
@@ -959,10 +1019,24 @@ void CiscoConfig::importP(ASt asno, IPAddr *addr,
 	   << " has no import policy for AS" << peerAS << endl;
 
    NormalExpression *ne;
+   NormalExpression done;
    int last = 0;
    for (; fa && !last; fa = itr.next()) {
       ne = NormalExpression::evaluate(fa->filter, peerAS);
-      last = print(ne, fa->action, IMPORT);
+
+      if (eliminateDupMapParts) {
+	 NormalExpression *ne2;
+	 ne2 = new NormalExpression(*ne);
+	 ne2->do_not();
+	 done.do_and(*ne2);
+
+	 if (! done.isEmpty())
+	    last = print(ne, fa->action, IMPORT);
+
+	 done.do_or(*ne);
+      } else
+	 last = print(ne, fa->action, IMPORT);
+
       delete ne;
    }
 
@@ -995,10 +1069,25 @@ void CiscoConfig::static2bgp(ASt asno, IPAddr *addr) {
 	   << " has no static2bgp policy for AS" << asno << endl;
 
    NormalExpression *ne;
+   NormalExpression done;
    int last = 0;
    for (; fa && !last; fa = itr.next()) {
       ne = NormalExpression::evaluate(fa->filter, asno);
-      last = print(ne, fa->action, EXPORT);
+
+      if (eliminateDupMapParts) {
+	 NormalExpression *ne2;
+	 ne2 = new NormalExpression(*ne);
+	 ne2->do_not();
+	 done.do_and(*ne2);
+	 delete ne2;
+
+	 if (! done.isEmpty())
+	    last = print(ne, fa->action, EXPORT);
+
+	 done.do_or(*ne);
+      } else
+	 last = print(ne, fa->action, EXPORT);
+
       delete ne;
    }
 
@@ -1209,7 +1298,7 @@ void CiscoConfig::importGroup(ASt asno, char * pset) {
 
    SymID psetID = symbols.symID(pset);
    // get matching import attributes
-   AutNumSelector<AttrImport> itr(autnum, "import", psetID, -1, NULL, NULL);
+   AutNumSelector<AttrImport> itr(autnum, "import", psetID, ~0, NULL, NULL);
    const FilterAction *fa = itr.first();
    if (! fa)
       cerr << "Warning: AS" << asno 
@@ -1218,7 +1307,7 @@ void CiscoConfig::importGroup(ASt asno, char * pset) {
    NormalExpression *ne;
    int last = 0;
    for (; fa && !last; fa = itr.next()) {
-      ne = NormalExpression::evaluate(fa->filter, -1);
+      ne = NormalExpression::evaluate(fa->filter, ~0);
       last = print(ne, fa->action, IMPORT);
       delete ne;
    }
@@ -1252,7 +1341,7 @@ void CiscoConfig::exportGroup(ASt asno, char * pset) {
 
    SymID psetID = symbols.symID(pset);
    // get matching export attributes
-   AutNumSelector<AttrExport> itr(autnum, "export", psetID, -1, NULL, NULL);
+   AutNumSelector<AttrExport> itr(autnum, "export", psetID, ~0, NULL, NULL);
    const FilterAction *fa = itr.first();
    if (! fa)
       cerr << "Warning: AS" << asno 
@@ -1261,7 +1350,7 @@ void CiscoConfig::exportGroup(ASt asno, char * pset) {
    NormalExpression *ne;
    int last = 0;
    for (; fa && !last; fa = itr.next()) {
-      ne = NormalExpression::evaluate(fa->filter, -1);
+      ne = NormalExpression::evaluate(fa->filter, ~0);
       last = print(ne, fa->action, EXPORT);
       delete ne;
    }
