@@ -6,8 +6,10 @@
 #include "autnum.hh"
 #include "route.hh"
 #include "util/trace.hh"
+#include "rpsl/object_log.hh"
 #include <memory.h>
 #include <malloc.h>
+#include <cstring>
 
 #ifndef BUFFER_SIZE
 #define BUFFER_SIZE            2047
@@ -17,7 +19,7 @@
 #define DATA_SEPARATOR         '\n'
 #define NO_OF_TRIAL_TRY		3
 
-BirdWhoisClient::BirdWhoisClient(void) : 
+BirdWhoisClient::BirdWhoisClient(void) : response(NULL),
   Socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)
 {
   Open(dflt_host, dflt_port, dflt_sources);
@@ -26,7 +28,7 @@ BirdWhoisClient::BirdWhoisClient(void) :
 BirdWhoisClient::BirdWhoisClient(const char *host, 
 				 const int port,
 				 const char *sources = dflt_sources) :
-  Socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)
+  response(NULL), Socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)
 {
   Open(host, port, sources);
 }
@@ -65,7 +67,11 @@ bool BirdWhoisClient::sendQuery(const char *pzcQuery, ...)
 {
   char pzcBuffer[BUFFER_SIZE + 1];
   char pzcFormat[512];
-  sprintf(pzcFormat, "-k -bird %s\r\n", pzcQuery);
+  if (sources && *sources) 
+     sprintf(pzcFormat, "-k -V RAToolSet -r -s %s %s\r\n", sources, pzcQuery);
+  else
+     sprintf(pzcFormat, "-k -V RAToolSet -r %s\r\n", pzcQuery);
+
   va_list ap;
   va_start(ap, pzcQuery);
   vsprintf(pzcBuffer, pzcFormat, ap);
@@ -73,8 +79,7 @@ bool BirdWhoisClient::sendQuery(const char *pzcQuery, ...)
 
   int iLen = strlen(pzcBuffer);
 
-  Trace(TR_WHOIS_QUERY) << getsock() 
-			<< " - Whois: WriteQuery " 
+  Trace(TR_WHOIS_QUERY) << "Whois: WriteQuery " 
 			<< pzcBuffer
 			<< " ("
 			<< iLen
@@ -86,80 +91,110 @@ bool BirdWhoisClient::sendQuery(const char *pzcQuery, ...)
   return true;
 }
 
-// Similiar to ripwhoisc.cc's read()
-bool BirdWhoisClient::getResponse(char *&text, int &len)
-{
-  char pzcBuffer[PROBE_BUFFER_SIZE];
-  char *pzcEor = "\n\n\n";
-  int iByteRead;
-  unsigned long ulSearchResult;
+// remove RIPE-style comments
+static void stripRipeComments(Buffer &buf) {
+   bool removedLines = false;
+   char *cont = buf.contents;
+   size_t size = buf.size;
 
-  text = NULL;
-  len = 0;
-
-  if (!cProbeBuffer.empty()  // if something in the probe buffer
-      && (ulSearchResult = cProbeBuffer.find(pzcEor)) <= cProbeBuffer.length())
-    ; // No statement here
-  else
-    {
-    int iNoofTry = 0;
-    while (1)
-      {
-      iByteRead = Socket::read(pzcBuffer, PROBE_BUFFER_SIZE);
-      if (iByteRead < 0)
-        if (++iNoofTry > NO_OF_TRIAL_TRY)
-          return false;
-	else
-	  continue;
-      cProbeBuffer.append(pzcBuffer, iByteRead);
-      if ((ulSearchResult = cProbeBuffer.find(pzcEor))<= cProbeBuffer.length())
-	break;
+   while (size >= 2 && *cont == '%') {
+      while (size >= 2 && *cont == '%') {
+	 // cengiz changed below to size - 1 
+	 // to make sure we received enough to check the stripped comment
+	 // is followed by  either another comment or by a blank line
+	 // if not we are in trouble...
+	 char *eol = (char *)memchr(cont, '\n', size-1);
+	 if (eol == NULL)
+	    // no EOL, no comment to strip (yet)
+	    goto out;
+	 assert(*(eol+1) == '\n' || *(eol+1) == '%');
+	 size -= (eol-cont)+1;
+	 cont = eol + 1;
+	 removedLines = true;
       }
-    }
+      if (*cont == '\n') {
+	 size--;
+	 cont++;
+      }
+   }
 
-  if (ulSearchResult == 0) 
-    {
-    cProbeBuffer.remove(3);
-    return false;
-    }
+ out:
+   if (removedLines)
+      buf.flush(buf.size - size);
+}
 
-  // At least found one \n\n\n
-  ulSearchResult++;  // the first \n belongs to the data itself
+bool BirdWhoisClient::getResponse(char *&text, int &len) {
+   int bytesRead;
+   bool found = false;
+   unsigned int endOfQuery;
 
-  // Try to get rid of leading \n
-  while (*(cProbeBuffer.c_str()) == '\n') 
-    {
-    cProbeBuffer.remove(1);
-    ulSearchResult--;
-    }
+   if (!response) 
+      // there may be some left over data after the previous query's response
+      response = new Buffer;
 
-  if (ulSearchResult <= 0) return false;  // Some sort of error
+   endOfQuery = 0;
+   do {
+      if (endOfQuery >= 3)
+	 endOfQuery -= 3;
+      for (; 
+	   endOfQuery + 2 < response->size && !found;
+	   endOfQuery++)
+	 if (response->contents[endOfQuery] == '\n' 
+	     && response->contents[endOfQuery+1] == '\n' 
+	     && response->contents[endOfQuery+2] == '\n')
+	    found = true;
 
-  len = ulSearchResult;
-  text = (char *)malloc(ulSearchResult + 1);
-  strncpy(text, cProbeBuffer.c_str(), ulSearchResult);
-  text[ulSearchResult] = 0;
-  cProbeBuffer.remove(ulSearchResult + 2);
+      if (found)
+	 break;
 
-  Trace(TR_WHOIS_RESPONSE) << getsock()
-			   << " - Whois: Response " 
-			   << '['
-			   << text 
-			   << ']'
-			   << endl;
+      if (response->capacity - response->size <= 0)
+	 response->extend();
+      bytesRead = Socket::read(response->contents + response->size, 
+			       response->capacity - response->size);
+      response->size += bytesRead;
 
-  return true;
+      if (bytesRead <= 0) {
+	 delete response;
+	 response=NULL;
+	 return false;
+      }
+   } while (!found);
+
+   Buffer *result = response;
+   if (endOfQuery+2 == response->size)
+      response = NULL;
+   else { // there is stuff after the response
+      int fullSize = response->size;
+      result->size = endOfQuery+2;
+      response = new Buffer(fullSize - result->size);
+      response->append(result->contents+result->size, response->capacity);
+   }
+
+   stripRipeComments(*result);
+   if (result->size == 2) { // we got 2 \n's
+      assert(*result->contents == '\n');
+      assert(*(result->contents+1) == '\n');
+      delete response;
+      response=NULL;
+      return false;
+   }
+
+   Trace(TR_WHOIS_RESPONSE).form("WhoisResponse[%.*s]\n",
+				 result->size, result->contents);
+   text = result->contents;
+   len  = result->size;
+   return true;
 }
 
 bool BirdWhoisClient::getAutNum(char *as,          char *&text, int &len)
 {
-  if (!sendQuery("aut-num%c%s", QUERY_FIELD_SEPARATOR, as)) return false;
+  if (!sendQuery("-T aut-num %s", as)) return false;
   if (!getResponse(text, len)) return false;
   return true;
 }
 
 bool BirdWhoisClient::getSet(SymID sname, char *clss, char *&text, int &len) {
-   if (!sendQuery("%s%c%s", clss, QUERY_FIELD_SEPARATOR, sname)) 
+   if (!sendQuery("-T %s %s", clss, sname)) 
       return false;
    if (!getResponse(text, len)) 
       return false;
@@ -168,91 +203,124 @@ bool BirdWhoisClient::getSet(SymID sname, char *clss, char *&text, int &len) {
 
 bool BirdWhoisClient::getRoute(char *rt, char *as, char *&text, int &len)
 {
-  if (!sendQuery("route%c%s%c%s", 
-		 QUERY_FIELD_SEPARATOR, rt, 
-		 QUERY_FIELD_SEPARATOR, as)) 
-    return false;
-  if (!getResponse(text, len)) return false;
-  return true;
+   if (strcasecmp(protocol, "ripe")) {
+      if (!sendQuery("-T route %s@%s", rt, as)) 
+	 return false;
+      if (!getResponse(text, len)) 
+	 return false;
+      return true;
+   }
+   // ripev3 support
+   // ripe does not have a query to get a specific route object given its key
+   // here is a work around...
+
+   char *response;
+   int l;
+   // ripev3 support
+   if (!sendQuery("-x -T route %s", rt)) 
+      return false;
+
+   if (!getResponse(response, l)) 
+      return false;
+   
+   List<Object> objs;
+   Buffer respBuf(response, l);
+   objectLog(respBuf, objs);
+   int asn = atoi(as+2);
+
+   for (ListIterator<Object> o(objs); o; ++o) {
+      AttrGenericIterator<ItemASNO> i(o, "origin"); 
+      if (i && i->asno == asn) {
+	 text = (char *) malloc(o->size);
+	 strncpy(text, o->contents, o->size);
+	 len  = o->size;
+	 // we may have a leak here
+	 free(response);
+	 return true;
+      }
+   }
+
+   return false;
 }
 
 bool BirdWhoisClient::getInetRtr(SymID inetrtr,    char *&text, int &len)
 {
-  if (!sendQuery("inet-rtr%c%s", QUERY_FIELD_SEPARATOR, inetrtr))
+  if (!sendQuery("-T inet-rtr %s", inetrtr))
     return false;
   if (!getResponse(text, len)) return false;
   return true;
 }
 
 bool BirdWhoisClient::expandAS(char *as, PrefixRanges *result) {
-  if (!sendQuery("route%corigin%c%s",
-		 QUERY_FIELD_SEPARATOR,
-		 QUERY_FIELD_SEPARATOR, as)) 
+  if (!sendQuery("-K -T route -i origin %s", as)) 
     return false;
   char *pzcKeys;
   int iKeyLen;
   if (getResponse(pzcKeys, iKeyLen))
     {
-    /* The response would look like the following:
-    route@204.140.128.0/17@AS226
-    route@192.45.0.0/16@AS226
-    route@192.41.208.0/24@AS226
-    route@204.115.168.0/21@AS226
-      :
-    */
-    char *p, *p2, *fence = pzcKeys + iKeyLen;
-    for (p = pzcKeys; 
-	 (p = strchr(p, QUERY_FIELD_SEPARATOR)) != 0 && p < fence; )
-      {
-      p++;
-      if ((p2 = strchr(p, QUERY_FIELD_SEPARATOR)) == 0) break;
-      *p2 = 0;
-      result->add_high(PrefixRange(p));
-      //cout << ">>" << p << endl;
-      if ((p = strchr(p2 + 1, DATA_SEPARATOR)) == 0) break;
-      p++;
-      }
+       assert(*(pzcKeys+iKeyLen-1) == '\n');
+       *(pzcKeys+iKeyLen) = 0;
+       char *p = pzcKeys;
+       char *prefix;
+       while (p && p < pzcKeys + iKeyLen) {
+	  p = strstr(p, "route");
+	  if (!p)
+	     break;
+	  p += 6; // we skip one extra character for ':' or '@' depending on the protocol
+	  while (*p && isspace(*p))
+	     p++;
+	  prefix = p;
+	  while (*p && (!isspace(*p) && *p != '@'))
+	     p++;
+	  *p = 0;
+	  p++;
+	  result->add_high(prefix);
+       }
     free(pzcKeys);
+    return true;
     }
-  return true;  
+  return false;  
 }
 
-bool BirdWhoisClient::getIndirectMembers(const Object *o, char *query, 
-					 void (*collect)(void *, const Object *), void *collectArg) {
-   AttrGenericIterator<ItemWORD> mbrs_by_ref(o, "mbrs-by-ref");
-   if (! mbrs_by_ref)
-      return false;
-
-   if (!sendQuery(query))
-      return false;
-
-   char *response, *response2;
-   int  len, len2;
+bool BirdWhoisClient::getIndirectMembers(char *setName,
+					  const Object *o, char *query, 
+					  void (*collect)(void *, const Object *), void *collectArg) {
+   char finalQuery[1024];
    bool isAny = false;
+   if (!strcasecmp(setName, "rs-any") || !strcasecmp(setName, "as-any"))
+      isAny = true;
+
+   AttrGenericIterator<ItemWORD> mbrs_by_ref(o, "mbrs-by-ref");
+   if (!isAny && ! mbrs_by_ref)
+      return false;
+
+   // if mbrs_by_ref contains any, then we only needs keys
+   for (mbrs_by_ref.first(); mbrs_by_ref && !isAny; mbrs_by_ref++)
+      isAny = ! strcasecmp(mbrs_by_ref()->word, "any");
+
+   if (isAny) 
+      strcpy(finalQuery, "-K ");
+   else
+      finalQuery[0] = 0;
+   strcat(finalQuery, query);
+      
+   if (!sendQuery(finalQuery))
+      return false;
+
+   char *response;
+   int  len;
    if (!getResponse(response, len))
       return false;
 
-   // if mbrs_by_ref contains any, then we are done
-   for (mbrs_by_ref.first(); mbrs_by_ref; mbrs_by_ref++)
-      if (! strcasecmp(mbrs_by_ref()->word, "any")) {
-	 isAny = true;
-	 break;
-      }
+   Buffer respBuf(response, len);
+   List<Object> objs;
+   objectLog(respBuf, objs);
 
-   // We need to omit members with wrong maintainers
-   for (char *p = response; 
-	p < response + len; 
-	p = strchr(p, DATA_SEPARATOR), p++)
-      if (sendQuery(p)) {
-	 if (getResponse(response2, len2)) {
-	    Buffer b(response2, len2);
-	    Object *mbr = new Object(b);
-	    if (isAny || isIndirectMember(mbr, mbrs_by_ref))
-	       (*collect)(collectArg, mbr);
-	    delete mbr;
-	    // NO need to free response2
-	 }
-      } 
+   for (ListIterator<Object> o(objs); o; ++o)
+      if (isAny || isIndirectMember(o, mbrs_by_ref))
+	 (*collect)(collectArg, o);
+
+   free(response);
 }
 
 bool BirdWhoisClient::expandRSSet(SymID sname, PrefixRanges *result) {
@@ -262,9 +330,12 @@ bool BirdWhoisClient::expandRSSet(SymID sname, PrefixRanges *result) {
      expandItem(pt, result);
 
   char buffer[1024];
-  sprintf(buffer, "route%cmember-of%c%s", 
-	  QUERY_FIELD_SEPARATOR, QUERY_FIELD_SEPARATOR, sname);
-  getIndirectMembers(set, buffer, collectPrefix, result);
+  // ripev3 support
+  if (!strcasecmp(protocol, "ripe") && !strcasecmp(sname, "rs-any")) 
+     strcpy(buffer, "-T route -M 0.0.0.0/0");
+  else
+     sprintf(buffer, "-T route -i member-of %s", sname);
+  getIndirectMembers(sname, set, buffer, collectPrefix, result);
 
   return true;
 }
@@ -283,9 +354,8 @@ bool BirdWhoisClient::expandASSet(SymID sname, SetOfUInt *result) {
        assert(false) ; ;
 
   char buffer[1024];
-  sprintf(buffer, "aut-num%cmember-of%c%s", 
-	  QUERY_FIELD_SEPARATOR, QUERY_FIELD_SEPARATOR, sname);
-  getIndirectMembers(set, buffer, collectASNO, result);
+  sprintf(buffer, "-T aut-num -i member-of %s", sname);
+  getIndirectMembers(sname, set, buffer, collectASNO, result);
 
   return true;
 }
@@ -297,16 +367,14 @@ bool BirdWhoisClient::expandRtrSet(SymID sname, PrefixRanges *result) {
      expandItem(pt, result);
 
   char buffer[1024];
-  sprintf(buffer, "inet-rtr%cmember-of%c%s", 
-	  QUERY_FIELD_SEPARATOR, QUERY_FIELD_SEPARATOR, sname);
-  getIndirectMembers(set, buffer, collectIfAddr, result);
+  sprintf(buffer, "-T inet-rtr -i member-of %s", sname);
+  getIndirectMembers(sname, set, buffer, collectIfAddr, result);
 
   return true;
 }
 
 int BirdWhoisClient::getSourceOrigin(char *&buffer, const char *rt)
 {
-  char *pzcReply;
   querySourceOrigin(rt);
   return getSourceOrigin(buffer); 
 }
@@ -319,15 +387,14 @@ int BirdWhoisClient::getSourceOrigin(char *&buffer)
   if (getResponse(pzcResult, iResultLen)) 
     {
     // More and complete route objects are here
-    Buffer b(pzcResult);  // pass ownership to class Buffer
+    Buffer b(pzcResult, iResultLen);  // pass ownership to class Buffer
     List<Object> cObjects;
-    extern void objectLog(Buffer &buf, List<Object> *log);
-    objectLog(b, &cObjects);
-    for (ListIterator<Object> itrObject(cObjects); itrObject; itrObject++)
-      if (strcmp(itrObject()->type->name, "route") == 0)
+    objectLog(b, cObjects);
+    for (ListIterator<Object> itrObject(cObjects); itrObject; ++itrObject)
+      if (itrObject->type && strcmp(itrObject->type->name, "route") == 0)
 	{
-	AttrGenericIterator<ItemASNO> itrOrigin(itrObject(), "origin");
-	AttrGenericIterator<ItemWORD> itrSource(itrObject(), "source");
+	AttrGenericIterator<ItemASNO> itrOrigin(itrObject, "origin");
+	AttrGenericIterator<ItemWORD> itrSource(itrObject, "source");
 	if (itrOrigin && itrSource)
 	  cBuffer.append("%s AS%d\n", itrSource()->word, itrOrigin()->asno);
 	}
@@ -344,16 +411,11 @@ int BirdWhoisClient::getSourceOrigin(char *&buffer)
 
 void BirdWhoisClient::querySourceOrigin(const char *rt)
 {
-  // Dereference the query by prepending a '*'
-  // Otherwise, it would only return keys. 
-  // An extra query would be needed in order to get the real object!
-  sendQuery("*route%croute%c%s", 
-	    QUERY_FIELD_SEPARATOR, QUERY_FIELD_SEPARATOR, rt);
-}
-
-bool BirdWhoisClient::readReady(void)
-{
-  return Socket::readReady();
+   // ripev3 support
+   if (!strcasecmp(protocol, "ripe"))
+      sendQuery("-a -x -T route %s", rt);
+   else
+      sendQuery("-a -T route -i route %s", rt);
 }
 
 ASt BirdWhoisClient::getOrigin(char *format, ...)
@@ -364,9 +426,13 @@ ASt BirdWhoisClient::getOrigin(char *format, ...)
   vsprintf(buffer, format, ap);
   va_end(ap);
 
-  if (!sendQuery("<route%croute%c%s", 
-		 QUERY_FIELD_SEPARATOR, QUERY_FIELD_SEPARATOR, buffer))
-    return 0;
+  // ripev3 support
+  if (!strcasecmp(protocol, "ripe")) {
+     if (!sendQuery("-L -x -T route %s", buffer))
+	return 0;
+  } else
+     if (!sendQuery("-L -T route -i route %s", buffer))
+	return 0;
 
   ASt tResult = 0;
   char *pzcResult;
