@@ -1,4 +1,4 @@
-//  $Id: rawhoisc.cc 1.11.1.15.1.7 Mon, 22 May 2000 14:07:39 -0700 cengiz $
+//  $Id: rawhoisc.cc 1.11.1.15.1.9 Tue, 20 Feb 2001 08:11:55 -0800 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -41,6 +41,9 @@
 
 
 extern "C" {
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -55,9 +58,15 @@ extern "C" {
 #include "rpsl/object_log.hh"
 
 extern "C" {
+#ifndef HAVE_DECL_CONNECT
 extern int connect(...);
+#endif // HAVE_DECL_CONNECT
+#ifndef HAVE_DECL_SOCKET
 extern int socket(...);
+#endif // HAVE_DECL_SOCKET
+#ifndef HAVE_DECL_GETHOSTBYNAME
 extern struct hostent *gethostbyname(...);
+#endif // HAVE_DECL_GETHOSTBYNAME
 
 #ifdef DEFINE_HTONS_WITH_ELLIPSIS
 extern u_short htons(...);
