@@ -24,7 +24,7 @@
  */
 
 //
-//  $Id: prtraceroute.cc 1.8.1.4 Wed, 26 May 1999 11:32:22 -0700 cengiz $
+//  $Id: prtraceroute.cc 1.8.1.7 Mon, 09 Apr 2001 10:38:04 -0700 cengiz $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -345,6 +345,7 @@ int mFlag(char *dst, char *key, char *nextArg) {
       fprintf (stderr, "TTL must be greater than one.");
       exit (1);
    }
+   return 1;
 }
 
 //------------------------------------------------------------
@@ -355,7 +356,7 @@ int mFlag(char *dst, char *key, char *nextArg) {
 int sFlag(char *dst, char *key, char *nextArg) {
    if (nextArg) sourceif = nextArg;
    fprintf (stderr, "Sorry '-s source' not supported yet'\n");
-   return 0;
+   return 1;
 }
 
 //------------------------------------------------------------
@@ -473,7 +474,7 @@ ttlProbe::ttlProbe(char *dst, int sport, int dport,
 
 #ifdef IP_HDRINCL    
     setsockopt (IPPROTO_IP, IP_HDRINCL, (char *) &on, sizeof (on));
-#endif IP_HDRINCL
+#endif // IP_HDRINCL
     init_udp (sport, dport);
     if (error())
 	error.die ("ttlProbe: ");
@@ -650,7 +651,7 @@ Path::addHop(ipAddr *ip, int i) {
 //------------------------------------------------------------
 ASt 
 Path::getHopAS(int ttl) {
-  if (ttl <= last_ttl)
+  if (ttl <= last_ttl && hops[ttl])
     return hops[ttl]->getAS();
   else
     return 0;
@@ -696,7 +697,7 @@ Path::process_policies()  {
   
   //--- Policy Analysis of the Path Taken
 
-  //--- analyse from destination to source
+  //--- analyze from destination to source
   if (getHopAS(last_ttl) == destination.getAS()) 
     strcpy (imports[last_ttl], "destination");
   else
