@@ -1,4 +1,4 @@
-/* $Id: re2dfa.c 1.5.2.6 Wed, 26 May 1999 15:55:50 -0700 cengiz $
+/* $Id: re2dfa.c 1.5.2.9 Wed, 14 Mar 2001 22:40:22 -0800 cengiz $
  */
 /*
  * RSd release 0.1:
@@ -46,7 +46,7 @@
  */
 
 struct _rd_bit {
-    unsigned char	rb_offset;	/* Offset into byte array */
+    unsigned short	rb_offset;	/* Offset into byte array */
     unsigned char	rb_mask;	/* Mask within byte */
     unsigned char	rb_alloc;	/* 1 if that bit has been allocated */
     struct _rd_state	*rb_state;	/* State to which bit is allocated */
@@ -126,7 +126,7 @@ rd_init()
     if (first) {
 	first = 0;
 	for (i = 0; i < RD_MAXASPSTATES; i++) {
-	    rd_bit[i].rb_offset = (unsigned char) i / RD_NBBY;
+	    rd_bit[i].rb_offset = (unsigned short) i / RD_NBBY;
 	    rd_bit[i].rb_mask = (unsigned char) 0x80 >> (i % RD_NBBY);
 	}
     }
@@ -237,7 +237,9 @@ rd_insert_range(rd_rhead *rlist,
 	if (((rd_range *) (rq->rq_self))->ra_low <= rtoins->ra_low) {
 	    pra = (rd_range *) (rq->rq_self);
 	    if (pra->ra_high >= rtoins->ra_low) {
-		pra->ra_high = rtoins->ra_high;
+                if (pra->ra_high < rtoins->ra_high) {
+                   pra->ra_high = rtoins->ra_high;
+                }
 		RD_FREE_RANGE(rtoins);
 		rtoins = pra;
 	    } else {
@@ -402,7 +404,7 @@ rd_insert_arc(rd_dq  *alist,		/* List of arcs */
      * arc. Otherwise, we start inserting arc at the head of the list.
      */
 
-    arc = *arctoins; 		/* Struct copy */
+    memcpy(&arc, arctoins, sizeof(arc)); /* Struct copy */
     if (RDQ_ISEMPTY(alist)) {
 	RD_ALLOC_ARC(nra, arc.ra_low, arc.ra_high);
 	RD_OR_BITS(nra->ra_mask, arc.ra_mask);
@@ -670,14 +672,14 @@ rd_alternate(rd_fm *fm1,			/* To this NFA.. */
     rd_arc		*ra;			/* Current arc in loop */
 
     if (fm1->bolexp)
-       rd_make_bol(fm2);
+       fm2 = rd_make_bol(fm2);
     else if (fm2->bolexp) 
-	  rd_make_bol(fm1);
+	  fm1 = rd_make_bol(fm1);
 
     if (fm1->eolexp)
-       rd_make_eol(fm2);
+       fm2 = rd_make_eol(fm2);
     else if (fm2->eolexp) 
-	  rd_make_eol(fm1);
+	  fm1 = rd_make_eol(fm1);
 
     /*
      * Insert each arc from start2 into start1.
@@ -1335,6 +1337,7 @@ rd_minimize(rd_fm *dfa)
     i = 0;
     RDQ_LIST_START(&(dfa->rf_states), dfa, rs, rd_state) {
 	rs->rs_bit = i++;
+	rs->rs_flags &= ~RSF_DELETE;
     } RDQ_LIST_END(&(dfa->rf_states), dfa, rs, rd_state);
     RD_ALLOC_NOTSAME(i); 
 
